"""
Mock `gpsdshm.Shm` for testing without a real GPS.

Markus Juenemann, 04-Feb-2016

"""

import time
from gpsdshm.shm import MAXCHANNELS

class MockFix(object):
    def __init__(self):
        self.time = time.time()
        self.mode = 3
        self.ept = 0.005
        self.latitude = -36.75859333333334
        self.epy = 14.853882022256698
        self.longitude = 143.99911
        self.epx = 10.040781879264545
        self.altitude = 68.2
        self.epv = 19.55
        self.track = 44.45
        self.epd = float('nan')
        self.speed = 0.3446777748
        self.eps = 29.707764044513397
        self.climb = 0.0
        self.epc = float('nan')


class MockDop(object):
    def __init__(self):
        self.xdop = 0.6693854586176363
        self.ydop = 0.9902588014837799
        self.pdop = 1.17
        self.hdop = 0.99
        self.vdop = 0.82
        self.tdop = 1.2548625723457216
        self.gdop = 2.4342743978108503


class MockSatellite(object):
    def __init__(self, prn):
        self.ss = 0.0 
        self.prn = self.PRN = prn
        self.used = True
        self.elevation = prn
        self.azimuth = prn
        

class MockShm(object):

    def __init__(self):
        self.online = time.time()
        self.status = True
        self.fix_time = time.time()
        self.satellites_visible = 8
        self.fix = MockFix()
        self.dop = MockDop()
        self.skyview_time = float('nan')
        self.satellites = [MockSatellite(prn) for prn in range(MAXCHANNELS)]
         
    



"""
Python interface to GPSd Shared Memory.

"""

import gpsdshm.shm


class Fix(object):

    def __init__(self, shm):
        self.shm = shm

    time = property(lambda self: gpsdshm.shm.get_fix_time(self.shm))
    mode = property(lambda self: gpsdshm.shm.get_fix_mode(self.shm))
    ept = property(lambda self: gpsdshm.shm.get_fix_ept(self.shm))
    latitude = property(lambda self: gpsdshm.shm.get_fix_latitude(self.shm))
    epy = property(lambda self: gpsdshm.shm.get_fix_epy(self.shm))
    longitude = property(lambda self: gpsdshm.shm.get_fix_longitude(self.shm))
    epx = property(lambda self: gpsdshm.shm.get_fix_epx(self.shm))
    altitude = property(lambda self: gpsdshm.shm.get_fix_altitude(self.shm))
    epv = property(lambda self: gpsdshm.shm.get_fix_epv(self.shm))
    track = property(lambda self: gpsdshm.shm.get_fix_track(self.shm))
    epd = property(lambda self: gpsdshm.shm.get_fix_epd(self.shm))
    speed = property(lambda self: gpsdshm.shm.get_fix_speed(self.shm))
    eps = property(lambda self: gpsdshm.shm.get_fix_eps(self.shm))
    climb = property(lambda self: gpsdshm.shm.get_fix_climb(self.shm))
    epc = property(lambda self: gpsdshm.shm.get_fix_epc(self.shm))


class Dop(object):

    def __init__(self, shm):
        self.shm = shm

    xdop = property(lambda self: gpsdshm.shm.get_dop_xdop(self.shm))
    ydop = property(lambda self: gpsdshm.shm.get_dop_ydop(self.shm))
    pdop = property(lambda self: gpsdshm.shm.get_dop_pdop(self.shm))
    hdop = property(lambda self: gpsdshm.shm.get_dop_hdop(self.shm))
    vdop = property(lambda self: gpsdshm.shm.get_dop_vdop(self.shm))
    tdop = property(lambda self: gpsdshm.shm.get_dop_tdop(self.shm))
    gdop = property(lambda self: gpsdshm.shm.get_dop_gdop(self.shm))


class Satellite(object):
    def __init__(self, ss, used, prn, elevation, azimuth):
        self.ss = self.snr = ss
        self.used = used and True
        self.prn = self.PRN = prn
        self.elevation = elevation
        self.azimuth = azimuth


class Satellites(object):
    """List of `GpsdShmSatellite` instances.

    """
    
    def __init__(self, shm):
        self.shm = shm

    def __getitem__(self, index):

        if index > gpsdshm.shm.MAXCHANNELS-1:
            raise IndexError

        ss = gpsdshm.shm.get_satellite_ss(self.shm, index)
        used = gpsdshm.shm.get_satellite_used(self.shm, index) == True
        prn = gpsdshm.shm.get_satellite_prn(self.shm, index)
        elevation = gpsdshm.shm.get_satellite_elevation(self.shm, index)
        azimuth = gpsdshm.shm.get_satellite_azimuth(self.shm, index)

        return Satellite(ss, used, prn, elevation, azimuth)


class Device(object):
    def __init__(self, path, flags, driver, subtype, activated,
                 baudrate, stopbits, parity, cycle, mincycle,
                 driver_mode):
        pass


class Devices(object):
    """List of `GpsdShmDevice` (singular) instances.

    """
    
    def __init__(self, shm):
        self.shm = shm

    def __getitem__(self, index):
        return None


class Shm(object):
    """GPSd Shared Memory.

    """

    def __init__(self):

        self.shm = gpsdshm.shm.shm_get()
        if self.shm == None:
            raise OSError('GPSd shared memory error: %s' % (self.shm, gpsdshm._error))

        self.fix = Fix(self.shm)
        self.dop = Dop(self.shm)
        self.satellites = Satellites(self.shm)
        self.devices = Devices(self.shm)


    set = property(lambda self: gpsdshm.shm.get_set(self.shm))
    online = property(lambda self: gpsdshm.shm.get_online(self.shm))
    fd = property(lambda self: gpsdshm.shm.get_fd(self.shm))
    status = property(lambda self: gpsdshm.shm.get_status(self.shm) != 0)
    dev = device = property(lambda self: gpsdshm.shm.get_dev(self.shm))
    skyview_time = property(lambda self: gpsdshm.shm.get_skyview_time(self.shm))
    satellites_visible = property(lambda self: gpsdshm.shm.get_satellites_visible(self.shm))
    
    
    



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 2.0.10
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.



from sys import version_info
if version_info >= (2,6,0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_shm', [dirname(__file__)])
        except ImportError:
            import _shm
            return _shm
        if fp is not None:
            try:
                _mod = imp.load_module('_shm', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _shm = swig_import_helper()
    del swig_import_helper
else:
    import _shm
del version_info
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError(name)

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0



def shm_get():
  return _shm.shm_get()
shm_get = _shm.shm_get

def get_fix_time(*args):
  return _shm.get_fix_time(*args)
get_fix_time = _shm.get_fix_time

def get_fix_mode(*args):
  return _shm.get_fix_mode(*args)
get_fix_mode = _shm.get_fix_mode

def get_fix_ept(*args):
  return _shm.get_fix_ept(*args)
get_fix_ept = _shm.get_fix_ept

def get_fix_latitude(*args):
  return _shm.get_fix_latitude(*args)
get_fix_latitude = _shm.get_fix_latitude

def get_fix_epy(*args):
  return _shm.get_fix_epy(*args)
get_fix_epy = _shm.get_fix_epy

def get_fix_longitude(*args):
  return _shm.get_fix_longitude(*args)
get_fix_longitude = _shm.get_fix_longitude

def get_fix_epx(*args):
  return _shm.get_fix_epx(*args)
get_fix_epx = _shm.get_fix_epx

def get_fix_altitude(*args):
  return _shm.get_fix_altitude(*args)
get_fix_altitude = _shm.get_fix_altitude

def get_fix_epv(*args):
  return _shm.get_fix_epv(*args)
get_fix_epv = _shm.get_fix_epv

def get_fix_track(*args):
  return _shm.get_fix_track(*args)
get_fix_track = _shm.get_fix_track

def get_fix_epd(*args):
  return _shm.get_fix_epd(*args)
get_fix_epd = _shm.get_fix_epd

def get_fix_speed(*args):
  return _shm.get_fix_speed(*args)
get_fix_speed = _shm.get_fix_speed

def get_fix_eps(*args):
  return _shm.get_fix_eps(*args)
get_fix_eps = _shm.get_fix_eps

def get_fix_climb(*args):
  return _shm.get_fix_climb(*args)
get_fix_climb = _shm.get_fix_climb

def get_fix_epc(*args):
  return _shm.get_fix_epc(*args)
get_fix_epc = _shm.get_fix_epc

def get_dop_xdop(*args):
  return _shm.get_dop_xdop(*args)
get_dop_xdop = _shm.get_dop_xdop

def get_dop_ydop(*args):
  return _shm.get_dop_ydop(*args)
get_dop_ydop = _shm.get_dop_ydop

def get_dop_pdop(*args):
  return _shm.get_dop_pdop(*args)
get_dop_pdop = _shm.get_dop_pdop

def get_dop_hdop(*args):
  return _shm.get_dop_hdop(*args)
get_dop_hdop = _shm.get_dop_hdop

def get_dop_vdop(*args):
  return _shm.get_dop_vdop(*args)
get_dop_vdop = _shm.get_dop_vdop

def get_dop_tdop(*args):
  return _shm.get_dop_tdop(*args)
get_dop_tdop = _shm.get_dop_tdop

def get_dop_gdop(*args):
  return _shm.get_dop_gdop(*args)
get_dop_gdop = _shm.get_dop_gdop

def get_online(*args):
  return _shm.get_online(*args)
get_online = _shm.get_online

def get_status(*args):
  return _shm.get_status(*args)
get_status = _shm.get_status

def get_skyview_time(*args):
  return _shm.get_skyview_time(*args)
get_skyview_time = _shm.get_skyview_time

def get_satellites_visible(*args):
  return _shm.get_satellites_visible(*args)
get_satellites_visible = _shm.get_satellites_visible

def get_satellite_ss(*args):
  return _shm.get_satellite_ss(*args)
get_satellite_ss = _shm.get_satellite_ss

def get_satellite_used(*args):
  return _shm.get_satellite_used(*args)
get_satellite_used = _shm.get_satellite_used

def get_satellite_prn(*args):
  return _shm.get_satellite_prn(*args)
get_satellite_prn = _shm.get_satellite_prn

def get_satellite_elevation(*args):
  return _shm.get_satellite_elevation(*args)
get_satellite_elevation = _shm.get_satellite_elevation

def get_satellite_azimuth(*args):
  return _shm.get_satellite_azimuth(*args)
get_satellite_azimuth = _shm.get_satellite_azimuth
MAXCHANNELS = _shm.MAXCHANNELS
# This file is compatible with both classic and new-style classes.





# -*- coding: utf-8 -*-

NAME = 'gpsdshm'
VERSION = '0.1.1'
LICENSE = 'BSD License'
AUTHOR = 'Markus Juenemann'
EMAIL = 'markus@juenemann.net'
DESCRIPTION = 'Python interface to gpsd shared memory'
URL = 'https://github.com/mjuenema/python-gpsdshm'

from setuptools import setup, Extension
shm_module = Extension('gpsdshm._shm', sources=['gpsdshm/shm.c', 'gpsdshm/shm_wrap.c'],)

from os.path import join, dirname

readme = open(join(dirname(__file__), 'README.rst')).read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

requirements = open(join(dirname(__file__), 'requirements.txt')).read().split()
test_requirements = open(join(dirname(__file__), 'test_requirements.txt')).read().split()

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=readme + '\n\n' + history,
    author=AUTHOR,
    author_email=EMAIL,
    url=URL,
    packages=[
        NAME,
    ],
    package_dir={NAME: NAME},
    include_package_data=True,
    install_requires=requirements,
    ext_modules = [shm_module],
    py_modules = [NAME],
    license=LICENSE,
    zip_safe=False,
    keywords='gpsd, shared memory',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Topic :: Software Development :: Libraries',
        'Programming Language :: C',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



"""
Tests for gpsdshm.

These tests can be run against a real gpsd with shared memory 
or a mock-up. This is determined by whether the `shm.fix.latitude`
attribute returns a "real" value or 0.0. The mock-up version is
useful when running 'python setup.py test' or testing under tox
and there is no gpsd process running.

The threshholds for valid values are somewhat arbitrary with the
hope that they pass in most common scenarios. If testing against 
a real GPS, let the GPS settle-in for a while. Also don't run
the tests while at supersonic speed or at higher altitude than 
Mt Everest. They are guaranteed to fail!

Markus Juenemann, 29-Jan-2016

"""

import sys
import math

#from nose.tools import *

import gpsdshm
import time

gpsd_shm = None


def setup():
    global gpsd_shm

    gpsd_shm = gpsdshm.Shm()

    if gpsd_shm.fix.latitude != 0.0:
        sys.stderr.write('Using real gpsd data for tests...\n')
    else:
        sys.stderr.write('Using mock gpsd data for tests...\n')
        import _mock
        gpsd_shm = _mock.MockShm()


def test_gpsdshm():

    now = time.time()

    assert isinstance(gpsd_shm.online, (float))
    assert now-2 < gpsd_shm.online < now+2

    assert gpsd_shm.status is True

    assert isinstance(gpsd_shm.skyview_time, (float))
    assert now-2 < gpsd_shm.skyview_time < now+2 or math.isnan(gpsd_shm.skyview_time)

    assert isinstance(gpsd_shm.satellites_visible, (int))
    assert 0 < gpsd_shm.satellites_visible < 73

    assert isinstance(gpsd_shm.fix.time, (float))
    assert now-2 < gpsd_shm.fix.time < now+2

    assert gpsd_shm.fix.mode in [2,3]

    assert isinstance(gpsd_shm.fix.ept, (float))
    assert 0.0 < gpsd_shm.fix.ept < 0.1

    assert isinstance(gpsd_shm.fix.latitude, (float))
    assert -90.0 < gpsd_shm.fix.latitude < 90.0

    assert isinstance(gpsd_shm.fix.epy, (float))
    assert 1.0 < gpsd_shm.fix.epy < 100.0

    assert isinstance(gpsd_shm.fix.longitude, (float))
    assert -180.0 < gpsd_shm.fix.longitude < 180.0

    assert isinstance(gpsd_shm.fix.epx, (float))
    assert 1.0 < gpsd_shm.fix.epx < 100.0

    assert isinstance(gpsd_shm.fix.altitude, (float))
    assert -100.0 < gpsd_shm.fix.altitude < 12000.0

    assert isinstance(gpsd_shm.fix.epv, (float))
    assert 1.0 < gpsd_shm.fix.epv < 100.0

    assert isinstance(gpsd_shm.fix.track, (float))
    assert 0.0 <= gpsd_shm.fix.track <= 365.0

    assert isinstance(gpsd_shm.fix.epd, (float))
    assert 0.0 <= gpsd_shm.fix.epd <= 365 or math.isnan(gpsd_shm.fix.epd)

    assert isinstance(gpsd_shm.fix.speed, (float))
    assert 0.0 <= gpsd_shm.fix.speed <= 343.0

    assert isinstance(gpsd_shm.fix.eps, (float))
    assert 0.0 <= gpsd_shm.fix.eps <= 343.0

    assert isinstance(gpsd_shm.fix.climb, (float))
    assert -100.0 < gpsd_shm.fix.climb < 100.0

    assert isinstance(gpsd_shm.fix.epc, (float))
    assert -100.0 < gpsd_shm.fix.epc < 100.0 or math.isnan(gpsd_shm.fix.epc)

    assert isinstance(gpsd_shm.dop.xdop, (float))
    assert 0.0 < gpsd_shm.dop.xdop < 3.0

    assert isinstance(gpsd_shm.dop.ydop, (float))
    assert 0.0 < gpsd_shm.dop.ydop < 3.0

    assert isinstance(gpsd_shm.dop.pdop, (float))
    assert 0.0 < gpsd_shm.dop.pdop < 3.0

    assert isinstance(gpsd_shm.dop.hdop, (float))
    assert 0.0 < gpsd_shm.dop.hdop < 3.0

    assert isinstance(gpsd_shm.dop.vdop, (float))
    assert 0.0 < gpsd_shm.dop.vdop < 3.0

    assert isinstance(gpsd_shm.dop.tdop, (float))
    assert 0.0 < gpsd_shm.dop.tdop < 3.0

    assert isinstance(gpsd_shm.dop.gdop, (float))
    assert 0.0 < gpsd_shm.dop.gdop < 3.0

    for i in range(gpsdshm.shm.MAXCHANNELS):
        assert isinstance(gpsd_shm.satellites[i].ss, (float))
        assert 0.0 <= gpsd_shm.satellites[i].ss < 50.

        assert isinstance(gpsd_shm.satellites[i].used, (bool))

        assert isinstance(gpsd_shm.satellites[i].prn, (int))
        assert 0 <= gpsd_shm.satellites[i].prn

        assert isinstance(gpsd_shm.satellites[i].PRN, (int))
        assert gpsd_shm.satellites[i].prn == gpsd_shm.satellites[i].PRN

        assert isinstance(gpsd_shm.satellites[i].elevation, (int))
        assert 0 <= gpsd_shm.satellites[i].elevation <= 90

        assert isinstance(gpsd_shm.satellites[i].azimuth, (int))
        assert 0 <= gpsd_shm.satellites[i].azimuth , 360.0
